package ru.ulstu.paper.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.Errors;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import ru.ulstu.core.model.response.Response;
import ru.ulstu.paper.model.PaperDto;
import ru.ulstu.paper.model.PaperFilterDto;
import ru.ulstu.paper.model.PaperStatusDto;
import ru.ulstu.paper.service.PaperService;

import javax.validation.Valid;
import java.io.IOException;
import java.util.List;


@Controller()
@RequestMapping(value = "/papers")
public class PaperController {
    private final PaperService paperService;

    public PaperController(PaperService paperService) {
        this.paperService = paperService;
    }

    @GetMapping("/papers")
    public void getPapers(ModelMap modelMap) {
        modelMap.put("papers", paperService.findAllDto());
    }

    @GetMapping("/paper")
    public void getPapers(ModelMap modelMap, @RequestParam(value = "id") Integer id) {
        if (id != null && id > 0) {
            modelMap.put("paperDto", paperService.findOneDto(id));
        } else {
            modelMap.put("paperDto", new PaperDto());
        }
    }

    @PostMapping("/paper")
    public String save(@Valid PaperDto paperDto, Errors errors) throws IOException {
        if (errors.hasErrors()) {
            return "/papers/paper";
        }
        paperService.save(paperDto);
        return "redirect:/papers/papers";
    }

    @DeleteMapping("/{paper-id}")
    public Response<Boolean> delete(@PathVariable("paper-id") Integer paperId) throws IOException {
        paperService.delete(paperId);
        return new Response<>(true);
    }

    @GetMapping("/statuses")
    public Response<List<PaperStatusDto>> getPaperStatuses() {
        return new Response<>(paperService.getPaperStatuses());
    }

    @PostMapping("/filter")
    public Response<List<PaperDto>> filter(@RequestBody @Valid PaperFilterDto paperFilterDto) throws IOException {
        return new Response<>(paperService.filter(paperFilterDto));
    }
}
