package ru.ulstu.paper.model;

import org.hibernate.validator.constraints.NotEmpty;
import org.springframework.format.annotation.DateTimeFormat;
import ru.ulstu.user.model.UserDto;

import javax.validation.constraints.Future;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.Set;
import java.util.stream.Collectors;

import static ru.ulstu.core.util.StreamApiUtils.convert;

public class PaperDto {
    private Integer id;
    @NotEmpty
    @Size(min = 3, max = 100)
    private String title;
    private Paper.PaperStatus status;
    private Date createDate;
    private Date updateDate;
    @NotNull
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date deadlineDate;
    private String comment;
    private Boolean locked;
    private String tmpFileName;
    private Integer fileId;
    private String fileName;
    private Date fileCreateDate;
    private Set<UserDto> authors;

    public PaperDto() {
    }

    public PaperDto(Paper paper) {
        this.id = paper.getId();
        this.title = paper.getTitle();
        this.status = paper.getStatus();
        this.createDate = paper.getCreateDate();
        this.updateDate = paper.getUpdateDate();
        this.deadlineDate = paper.getDeadlineDate();
        this.comment = paper.getComment();
        this.locked = paper.getLocked();
        this.tmpFileName = null;
        this.fileId = paper.getFileData() == null ? null : paper.getFileData().getId();
        this.fileName = paper.getFileData() == null ? null : paper.getFileData().getName();
        this.fileCreateDate = paper.getFileData() == null ? null : paper.getFileData().getCreateDate();
        this.authors = convert(paper.getAuthors(), UserDto::new);
    }

    public Integer getId() {
        return id;
    }

    public String getTitle() {
        return title;
    }

    public Paper.PaperStatus getStatus() {
        return status;
    }

    public Date getCreateDate() {
        return createDate;
    }

    public Date getUpdateDate() {
        return updateDate;
    }

    public Date getDeadlineDate() {
        return deadlineDate;
    }

    public String getComment() {
        return comment;
    }

    public Boolean getLocked() {
        return locked;
    }

    public String getTmpFileName() {
        return tmpFileName;
    }

    public Integer getFileId() {
        return fileId;
    }

    public String getFileName() {
        return fileName;
    }

    public Date getFileCreateDate() {
        return fileCreateDate;
    }

    public Set<UserDto> getAuthors() {
        return authors;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void setStatus(Paper.PaperStatus status) {
        this.status = status;
    }

    public void setCreateDate(Date createDate) {
        this.createDate = createDate;
    }

    public void setUpdateDate(Date updateDate) {
        this.updateDate = updateDate;
    }

    public void setDeadlineDate(Date deadlineDate) {
        this.deadlineDate = deadlineDate;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public void setLocked(Boolean locked) {
        this.locked = locked;
    }

    public void setTmpFileName(String tmpFileName) {
        this.tmpFileName = tmpFileName;
    }

    public void setFileId(Integer fileId) {
        this.fileId = fileId;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public void setFileCreateDate(Date fileCreateDate) {
        this.fileCreateDate = fileCreateDate;
    }

    public void setAuthors(Set<UserDto> authors) {
        this.authors = authors;
    }

    public String getAuthorsString() {
        return authors
                .stream()
                .map(author -> author.getLastName() + author.getFirstName())
                .collect(Collectors.joining(", "));
    }
}
